"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;
var _debug = _interopRequireDefault(require("debug"));
var _core = require("@verdaccio/core");
var _memoryHandler = _interopRequireDefault(require("./memory-handler"));
function _interopRequireDefault(e) { return e && e.__esModule ? e : { default: e }; }
const debug = (0, _debug.default)('verdaccio:plugin:storage:local-memory');
const DEFAULT_LIMIT = 1000;
class LocalMemory extends _core.pluginUtils.Plugin {
  // @ts-ignore

  constructor(config, options) {
    super(config, options);
    this.config = config;
    this.limit = config.limit || DEFAULT_LIMIT;
    this.logger = options.logger;
    this.data = this._createEmtpyDatabase();
    this.path = '/';
    debug('start plugin');
  }
  init() {
    return Promise.resolve();
  }
  getSecret() {
    return Promise.resolve(this.data.secret);
  }
  setSecret(secret) {
    return new Promise(resolve => {
      this.data.secret = secret;
      resolve(null);
    });
  }
  async add(name) {
    return new Promise((resolve, reject) => {
      const {
        list
      } = this.data;
      if (list.length < this.limit) {
        if (list.indexOf(name) === -1) {
          list.push(name);
        }
        resolve();
      } else {
        this.logger.info({
          limit: this.limit
        }, 'Storage memory has reached limit of @{limit} packages');
        reject(new Error(`Storage memory has reached limit of ${this.limit} packages`));
      }
    });
  }

  // eslint-disable-next-line @typescript-eslint/no-unused-vars
  async search(query) {
    this.logger.warn('[verdaccio/memory]: search method not implemented, PR is welcome');
    return Promise.reject('not implemented');
  }
  async remove(name) {
    return new Promise(resolve => {
      const {
        list
      } = this.data;
      const item = list.indexOf(name);
      if (item !== -1) {
        list.splice(item, 1);
      }
      return resolve();
    });
  }
  async get() {
    debug('data list length %o', this.data?.list?.length);
    return Promise.resolve(this.data?.list);
  }
  getPackageStorage(packageInfo) {
    return new _memoryHandler.default(packageInfo, this.data.files, this.logger);
  }
  async hasTarball(/* fileName: string */
  ) {
    throw new Error('not  implemented');
  }
  async hasPackage() {
    return false;
  }
  _createEmtpyDatabase() {
    const list = [];
    const files = {};
    const emptyDatabase = {
      list,
      files,
      secret: ''
    };
    return emptyDatabase;
  }
  saveToken() {
    this.logger.warn('[verdaccio/memory][saveToken] save token has not been implemented yet');
    return Promise.reject(_core.errorUtils.getServiceUnavailable('method not implemented'));
  }
  deleteToken(user, tokenKey) {
    this.logger.warn({
      tokenKey,
      user
    }, '[verdaccio/memory][deleteToken] delete token has not been implemented yet @{user}');
    return Promise.reject(_core.errorUtils.getServiceUnavailable('method not implemented'));
  }
  readTokens() {
    this.logger.warn('[verdaccio/memory][readTokens] read tokens has not been implemented yet ');
    return Promise.reject(_core.errorUtils.getServiceUnavailable('method not implemented'));
  }
}
var _default = exports.default = LocalMemory;
//# sourceMappingURL=local-memory.js.map