"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;
var _debug = _interopRequireDefault(require("debug"));
var _memfs = require("memfs");
var _path = _interopRequireDefault(require("path"));
var _stream = require("stream");
var _promises = require("stream/promises");
var _core = require("@verdaccio/core");
var _utils = require("./utils");
function _interopRequireDefault(e) { return e && e.__esModule ? e : { default: e }; }
/* eslint-disable @typescript-eslint/no-unused-vars */

const debug = (0, _debug.default)('verdaccio:plugin:storage:memory-storage');
function fstatPromise(fd) {
  return new Promise((resolve, reject) => {
    _memfs.fs.fstat(fd, function (err, stats) {
      if (err) {
        return reject(err);
      }
      return resolve(stats);
    });
  });
}
function mkdirPPromise(fileName) {
  return new Promise((resolve, reject) => {
    _memfs.fs.mkdirp(_path.default.dirname(fileName), function (err) {
      if (err) {
        return reject(err);
      }
      return resolve(true);
    });
  });
}
class MemoryHandler {
  constructor(packageName, data, logger) {
    // this is not need it
    this.data = data;
    this.name = packageName;
    this.logger = logger;
    this.path = `/${packageName}`;
    debug('initialized');
  }
  async hasTarball(fileName) {
    throw new Error('not  implemented');
  }
  async hasPackage() {
    return false;
  }
  async deletePackage(pkgName) {
    delete this.data[pkgName];
    return;
  }
  removePackage() {
    return Promise.resolve();
  }
  async createPackage(name, value) {
    debug('create package %o', name);
    await this.savePackage(name, value);
  }
  async savePackage(name, value) {
    try {
      debug('save package %o', name);
      this.data[name] = (0, _utils.stringifyPackage)(value);
    } catch (err) {
      throw _core.errorUtils.getInternalError(err.message);
    }
  }
  async readPackage(name) {
    debug('read package %o', name);
    const json = this._getStorage(name);
    const isJson = typeof json === 'undefined';
    if (isJson) {
      throw _core.errorUtils.getNotFound();
    }
    return (0, _utils.parsePackage)(json);
  }
  async writeTarball(name) {
    const temporalName = `${this.path}/${name}`;
    debug('write tarball %o', temporalName);
    await mkdirPPromise(temporalName);
    const writeStream = _memfs.fs.createWriteStream(temporalName);
    return writeStream;
  }
  async readTarball(pkgName, {
    signal
  }) {
    const pathName = this._getStorage(pkgName);
    const passStream = new _stream.PassThrough();
    const readStream = (0, _stream.addAbortSignal)(signal, _memfs.fs.createReadStream(pathName));
    readStream.on('open', async function (fileDescriptor) {
      const stats = await fstatPromise(fileDescriptor);
      passStream.emit('content-length', stats?.size);
    });
    readStream.on('error', err => {
      passStream.emit('error', err);
    });
    await (0, _promises.pipeline)(readStream, passStream);
    return passStream;
  }

  // public readTarball(name: string): IReadTarball {
  //   const pathName = `${this.path}/${name}`;
  //   debug('read tarball %o', pathName);

  //   const readTarballStream: IReadTarball = new ReadTarball({});

  //   process.nextTick(function () {
  //     fs.stat(pathName, function (error, stats) {
  //       if (error && !stats) {
  //         return readTarballStream.emit('error', errorUtils.getNotFound());
  //       }

  //       try {
  //         const readStream = fs.createReadStream(pathName);

  //         readTarballStream.emit('content-length', stats?.size);
  //         readTarballStream.emit('open');
  //         readStream.pipe(readTarballStream);
  //         readStream.on('error', (error: VerdaccioError) => {
  //           readTarballStream.emit('error', error);
  //         });

  //         readTarballStream.abort = function (): void {
  //           readStream.destroy(errorUtils.getBadRequest('read has been aborted'));
  //         };
  //         return;
  //       } catch (err: any) {
  //         readTarballStream.emit('error', err);
  //         return;
  //       }
  //     });
  //   });

  //   return readTarballStream;
  // }

  _getStorage(name = '') {
    debug('get storage %o', name);
    return this.data[name];
  }
  async updatePackage(packageName, handleUpdate) {
    const json = this._getStorage(packageName);
    let pkg = (0, _utils.parsePackage)(json);
    const newManifest = await handleUpdate(pkg);
    return newManifest;
  }
}
var _default = exports.default = MemoryHandler;
//# sourceMappingURL=memory-handler.js.map